#ifndef LIBWCCL_OPS_MATCH_CONDITIONS_LONGEST_H
#define LIBWCCL_OPS_MATCH_CONDITIONS_LONGEST_H

#include <libwccl/ops/match/conditions/conjconditions.h>

namespace Wccl {

/**
 * Class for "longest" condition of match
 */
class Longest : public MatchCondition
{
public:
	Longest(const boost::shared_ptr<std::vector<ConjConditions> >& variants);

	/**
	 * @returns Name of the condition.
	 */
	std::string name() const {
		return "longest";
	}
	/**
	 * Applies the condition to the given execution context.
	 * Inner match variants are executed one by one to check which
	 * one is the longest one. If any match variants were found to be "true",
	 * the one that produced the longest match is returned with "true".
	 * If there was no "true" match variant, "false" is returned instead.
	 * If a match is found, the current sentence Position is increased
	 * as to point one token after all the matched tokens, otherwise
	 * it stays unchanged.
	 */
	MatchResult apply(const ActionExecContext& context) const;

	/**
	 * @returns String representation of the MatchCondition
	 */
	std::string to_string(const Corpus2::Tagset& tagset) const;

protected:
	/**
	 * Writes string representation of the MatchCondition to
	 * an output stream.
	 * @returns Stream written to.
	 * @note May be incomplete and/or containt internal info.
	 */
	std::ostream& write_to(std::ostream& ostream) const;
private:
	const boost::shared_ptr<std::vector<ConjConditions> > _variants;
};
} /* end ns Wccl */

#endif // LIBWCCL_OPS_MATCH_CONDITIONS_LONGEST_H
